#include "usart.h"

uint8_t usart_rxbuffer[USART_BUFFER_SIZE];
uint8_t usart_txbuffer[USART_BUFFER_SIZE];

struct {
	uint8_t prxbuffer:3;
	uint8_t ptxbuffer:3;
	uint8_t error:8;
} usart_status;

ISR(USART_RXC_vect)
{
	usart_rawget(UDR);
}

ISR(USART_TXC_vect)
{
	if ( usart_status.ptxbuffer > 0 ) 
		usart_rawput(usart_txpop());
}

uint8_t usart_init()
{
	UCSRB = (1<<RXCIE) | (1<<TXCIE) | (1<<RXEN)  | (1<<TXEN);  // Interrupts aktivieren
	UCSRC = (1<<URSEL) | (3<<UCSZ0);                           // Asynchron 8n1

	//UBRRH = (uint8_t) ( USART_UBRR_CALC( USART_BAUD_RATE, F_CPU ) >> 8 );
    UBRRL = (uint8_t)   USART_UBRR_CALC( USART_BAUD_RATE, F_CPU );

	return 0;
}

uint8_t usart_rawput(uint8_t ch)
{
	usart_txpush(ch);
}

uint8_t usart_rawget(uint8_t ch)
{

}

uint8_t usart_rawflush()
{
	UDR=3;
	UDR = usart_txpop();
}


uint8_t usart_rxpush(uint8_t ch)
{
	if (usart_status.prxbuffer < (USART_BUFFER_SIZE-1))
	{
		usart_rxbuffer[usart_status.prxbuffer++] = ch;
		usart_status.error = USART_OK;
		return 0;
	}
	else
	{
		usart_status.error = USART_RXBUFFER_FULL;
		return -1;
	}
}

uint8_t usart_rxpop()
{
	if (usart_status.prxbuffer > 0)
	{
		usart_status.error = USART_OK;
		return usart_rxbuffer[--usart_status.prxbuffer];
	}
	else
	{
		usart_status.error = USART_RXBUFFER_EMPTY;
		return -1;
	}
}

uint8_t usart_txpush(uint8_t ch)
{
	if (usart_status.ptxbuffer < (USART_BUFFER_SIZE-1))
	{
		usart_txbuffer[usart_status.ptxbuffer++] = ch;
		usart_status.error = USART_OK;
		return 0;
	}
	else
	{
		usart_status.error = USART_TXBUFFER_FULL;
		return -1;
	}
}

uint8_t usart_txpop()
{
	if (usart_status.ptxbuffer > 0)
	{
		usart_status.error = USART_OK;
		return usart_txbuffer[--usart_status.ptxbuffer];
	}
	else
	{
		usart_status.error = USART_TXBUFFER_EMPTY;
		return -1;
	}
}

